
{******************************************}
{                                          }
{             FastReport v3.0              }
{          Preview Page settings           }
{                                          }
{         Copyright (c) 1998-2004          }
{         by Alexander Tzyganenko,         }
{            Fast Reports Inc.             }
{                                          }
{******************************************}

unit frxPreviewPageSettings;

interface

{$I frx.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, frxClass
{$IFDEF Delphi6}
, Variants
{$ENDIF};

type
  TfrxPageSettingsForm = class(TForm)
    OKB: TButton;
    CancelB: TButton;
    SizeL: TGroupBox;
    WidthL: TLabel;
    HeightL: TLabel;
    cmL1: TLabel;
    cmL2: TLabel;
    WidthE: TEdit;
    HeightE: TEdit;
    SizeCB: TComboBox;
    OrientationL: TGroupBox;
    PortraitImg: TImage;
    LandscapeImg: TImage;
    PortraitRB: TRadioButton;
    LandscapeRB: TRadioButton;
    MarginsL: TGroupBox;
    LeftL: TLabel;
    TopL: TLabel;
    RightL: TLabel;
    BottomL: TLabel;
    cmL3: TLabel;
    cmL4: TLabel;
    cmL5: TLabel;
    cmL6: TLabel;
    MarginLeftE: TEdit;
    MarginTopE: TEdit;
    MarginRightE: TEdit;
    MarginBottomE: TEdit;
    OtherL: TGroupBox;
    ApplyToCurRB: TRadioButton;
    ApplyToAllRB: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure PortraitRBClick(Sender: TObject);
    procedure SizeCBClick(Sender: TObject);
    procedure WidthEChange(Sender: TObject);
  private
    { Private declarations }
    FUpdating: Boolean;
    FPage: TfrxReportPage;
    function GetNeedRebuild: Boolean;
  public
    { Public declarations }
    property Page: TfrxReportPage read FPage write FPage;
    property NeedRebuild: Boolean read GetNeedRebuild;
  end;


implementation

{$R *.DFM}

uses Printers, frxPrinter, frxUtils, frxRes;


function TfrxPageSettingsForm.GetNeedRebuild: Boolean;
begin
  Result := ApplyToAllRB.Checked;
end;

procedure TfrxPageSettingsForm.FormShow(Sender: TObject);
var
  i: Integer;
begin
  FUpdating := True;

  SizeCB.Items := frxPrinters.Printer.Papers;
  i := frxPrinters.Printer.PaperIndex(Page.PaperSize);
  if i = -1 then
    i := frxPrinters.Printer.PaperIndex(256);
  SizeCB.ItemIndex := i;

  WidthE.Text := frxFloatToStr(Page.PaperWidth / 10);
  HeightE.Text := frxFloatToStr(Page.PaperHeight / 10);
  PortraitRB.Checked := Page.Orientation = poPortrait;
  LandscapeRB.Checked := Page.Orientation = poLandscape;

  MarginLeftE.Text := frxFloatToStr(Page.LeftMargin / 10);
  MarginRightE.Text := frxFloatToStr(Page.RightMargin / 10);
  MarginTopE.Text := frxFloatToStr(Page.TopMargin / 10);
  MarginBottomE.Text := frxFloatToStr(Page.BottomMargin / 10);

  PortraitRBClick(nil);
  FUpdating := False;
end;

procedure TfrxPageSettingsForm.FormHide(Sender: TObject);
begin
  if ModalResult = mrOk then
  begin
    if PortraitRB.Checked then
      Page.Orientation := poPortrait else
      Page.Orientation := poLandscape;

    Page.PaperWidth := frxStrToFloat(WidthE.Text) * 10;
    Page.PaperHeight := frxStrToFloat(HeightE.Text) * 10;
    Page.PaperSize := frxPrinters.Printer.PaperNameToNumber(SizeCB.Text);

    Page.LeftMargin := frxStrToFloat(MarginLeftE.Text) * 10;
    Page.RightMargin := frxStrToFloat(MarginRightE.Text) * 10;
    Page.TopMargin := frxStrToFloat(MarginTopE.Text) * 10;
    Page.BottomMargin := frxStrToFloat(MarginBottomE.Text) * 10;

    Page.AlignChildren;
  end;
end;

procedure TfrxPageSettingsForm.PortraitRBClick(Sender: TObject);
begin
  PortraitImg.Visible := PortraitRB.Checked;
  LandscapeImg.Visible := LandscapeRB.Checked;
  SizeCBClick(nil);
end;

procedure TfrxPageSettingsForm.SizeCBClick(Sender: TObject);
var
  pOr: TPrinterOrientation;
  pNumber: Integer;
  pWidth, pHeight: Extended;
begin
  if FUpdating then Exit;
  FUpdating := True;

  with frxPrinters.Printer do
  begin
    pNumber := PaperNameToNumber(SizeCB.Text);
    pWidth := frxStrToFloat(WidthE.Text) * 10;
    pHeight := frxStrToFloat(HeightE.Text) * 10;
    if PortraitRB.Checked then
      pOr := poPortrait else
      pOr := poLandscape;

    if pNumber = 256 then
      SetViewParams(pNumber, pHeight, pWidth, pOr) else
      SetViewParams(pNumber, pWidth, pHeight, pOr);

    WidthE.Text := frxFloatToStr(PaperWidth / 10);
    HeightE.Text := frxFloatToStr(PaperHeight / 10);
  end;

  FUpdating := False;
end;

procedure TfrxPageSettingsForm.WidthEChange(Sender: TObject);
begin
  if not FUpdating then
    SizeCB.ItemIndex := 0;
end;

procedure TfrxPageSettingsForm.FormCreate(Sender: TObject);
begin
  frxResources.LocalizeForm(Self);
end;

end.
